'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var React = _interopDefault(require('react'));
var addons = require('@storybook/addons');
var broadcastChannel = require('broadcast-channel');

const ManagerContainer = ({ active, id, api, title, }) => {
    const channel = React.useMemo(() => new broadcastChannel.BroadcastChannel(`attach_docs_page_${title}`), []);
    React.useEffect(() => {
        const iframe = document.getElementById('storybook-preview-iframe');
        const wrapper = document.getElementById('storybook-preview-wrapper');
        if (iframe && iframe.contentDocument) {
            const updateDOM = () => {
                var _a;
                const story = api.getCurrentStoryData();
                channel.postMessage({ id: id, active, storyId: story === null || story === void 0 ? void 0 : story.id });
                const root = (_a = iframe === null || iframe === void 0 ? void 0 : iframe.contentDocument) === null || _a === void 0 ? void 0 : _a.getElementById('root');
                if (wrapper && root) {
                    if (active) {
                        root.style.setProperty('display', 'none');
                        wrapper.removeAttribute('hidden');
                    }
                    else {
                        root.style.removeProperty('display');
                    }
                }
            };
            if (!iframe.contentDocument.getElementById('root')) {
                iframe.onload = () => {
                    updateDOM();
                };
            }
            else {
                updateDOM();
            }
        }
    }, [active]);
    return null;
};

/* eslint-disable react/display-name */
const ADDON_ID = 'CUSTOM_PAGE_ADDON';
const registerPages = (configs) => {
    addons.addons.register(ADDON_ID, api => {
        configs.forEach(pageConfig => {
            const title = pageConfig.title;
            const key = title.toLowerCase();
            addons.addons.add(`${ADDON_ID}/${key}`, {
                type: addons.types.TAB,
                title,
                route: ({ storyId }) => `/${key}/${storyId}`,
                match: ({ viewMode }) => viewMode === key,
                render: ({ active }) => (React.createElement(ManagerContainer, { active: active, title: title, id: `controls-docs-page-${key}`, api: api })),
            });
        });
    });
};
new broadcastChannel.BroadcastChannel('custom_page_send_props').onmessage = ({ config, }) => {
    registerPages(config);
};
new broadcastChannel.BroadcastChannel('custom_page_request_props').postMessage({});
